/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#include "stdafx.h"
#include "CPluginWindow.h"
#include "WMTop10.h"
#include "Utils.h"

#include <vector>
using namespace std;

const int urlWidth = 260;
const int urlHeight = 15;

CPluginWindow::CPluginWindow(CWMTop10 *pPlugin)
:m_pPlugin ( pPlugin)
, m_nWidth(0)
, m_nHeight(0)
{
    
}
LRESULT CPluginWindow::OnEraseBackground(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled)
{
    // avoid erasing background to reduce flicker on resize
	return 1;
}

LRESULT CPluginWindow::OnCreate(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{

	//create fonts
	hTitleFont = MakeFont(_T("Verdana"), 24, FW_BOLD);
	hGroupTitleFont = MakeFont(_T("Verdana"), 12, FW_DEMIBOLD);
	hListFont = MakeFont(_T("Arial"), 11, FW_NORMAL);
	

	//create pens
	hGroupBorderPen = CreatePen(PS_SOLID, 2, RGB(0,100,255));
	hDividerPen = CreatePen(PS_SOLID, 1, RGB(200,200,220));

	//create brushes
	hBackgroundBrush = CreateSolidBrush( RGB(0, 25, 75) );

	//load text
	m_strUrl.LoadString(IDS_URL);
	m_strUrlDisplay.LoadString(IDS_AUTHOR);
	m_strUrlDisplay+=_T(" - ");
	m_strUrlDisplay+=m_strUrl;

	m_pList = m_pPlugin->GetSharedList();
	
	return 0;
}


LRESULT CPluginWindow::OnDestroy(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	//destroy brushes
	DeleteObject(hBackgroundBrush);

	//detroy fonts
	DeleteObject(hGroupTitleFont);
	DeleteObject(hListFont);
	DeleteObject(hTitleFont);

	//destroy pens
	DeleteObject(hGroupBorderPen);
	DeleteObject(hDividerPen);
	
	return 0;
}

LRESULT CPluginWindow::OnSize(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	m_nWidth  = LOWORD(lParam);
	m_nHeight = HIWORD(lParam);

	MoveUrl();

	return 0;
}
//moves URL static control to lower-right hand corner of screen
void CPluginWindow::MoveUrl()
{
	if (m_nWidth > 0 && m_nHeight > 0 )
	{
		
		urlRect.left = m_nWidth - urlWidth;
		urlRect.top = m_nHeight - urlHeight;
		urlRect.right = m_nWidth;
		urlRect.bottom = m_nHeight;

	}
}

LRESULT CPluginWindow::OnSetCursor(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled)
{
	POINT pt;
	GetCursorPos(&pt);
	::ScreenToClient(m_hWnd, &pt);
	//if in urlRect, change cursor to hand
	if (IsMouseOverUrl(pt))
	{
		SetCursor(LoadCursor(NULL, IDC_HAND));
		return TRUE;
	}
	else
	{
		SetCursor(LoadCursor(NULL, IDC_ARROW));
	}
	return FALSE;
}


LRESULT CPluginWindow::OnPaint(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled)
{
    PAINTSTRUCT ps;

    HDC hDC = BeginPaint(&ps);

	RECT    rc;
    GetClientRect(&rc);

   
    FillRect(hDC, &rc, hBackgroundBrush );

	if (m_pPlugin->ServerRunning())
	{
		//draw title
		DrawMainTitle(hDC);

		DrawGroups(hDC);

		DrawUrl(hDC);
	}
	else
	{
		CString strError;
		strError.LoadString(IDS_SERVERNOTRUNNING);
		
		DrawErrorMessage(hDC, strError);
	}
    
    
    EndPaint(&ps);
	return 0;
}

void CPluginWindow::DrawUrl(HDC hDC)
{
	::SelectObject(hDC, hListFont);
	::SetTextColor(hDC, RGB(200,200,220));
	::SetBkMode(hDC, TRANSPARENT);
	::DrawText(hDC, m_strUrlDisplay, m_strUrlDisplay.GetLength(),&urlRect,0);
}


void CPluginWindow::DrawErrorMessage(HDC hDC, const CString& strError)
{
	POINT ptStart = {100,100};
	COLORREF rgbTextColor = RGB(255,255,255);
	
	int nOldDC = SaveDC(hDC);
	HBRUSH hNullBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	SelectObject(hDC, hNullBrush);

	::SelectObject(hDC, hGroupTitleFont);
	::SetTextColor(hDC, rgbTextColor);
	::SetBkMode(hDC, TRANSPARENT);
	const int nMargin = 50;
	RECT destRect = {nMargin,m_nHeight/2-nMargin,m_nWidth-(nMargin*2), m_nHeight / 2};
	::DrawText(hDC, strError, strError.GetLength(), &destRect, DT_CENTER | DT_WORDBREAK);
	RestoreDC(hDC, nOldDC);
}

void CPluginWindow::DrawMainTitle(HDC hDC)
{
	const CString strTitle(_T("Windows Media Top 10"));
	POINT ptStart = {5,5};
	COLORREF rgbTextColor = RGB(255,255,255);
	
	int nOldDC = SaveDC(hDC);
	HBRUSH hNullBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	SelectObject(hDC, hNullBrush);

	::SelectObject(hDC, hTitleFont);
	::SetTextColor(hDC, rgbTextColor);
	::SetBkMode(hDC, TRANSPARENT);
	::TextOut(hDC, ptStart.x,ptStart.y,strTitle,strTitle.GetLength());

	RestoreDC(hDC, nOldDC);
}

void CPluginWindow::DrawGroups(HDC hDC)
{
	POINT ptUpperLeft = {15, 70};
	SIZE groupSize = {300, 170}; 
	SIZE spaceBetween = {15,25};
	int nDividerOffset = (int)(groupSize.cx * 0.75);

	POINT ptCurrentPt = ptUpperLeft;
	int numCategories = m_pList->GetNumCategories();
	for (int category = 0;category<numCategories; category++)
	{
		RECT groupRect = 
			{
				ptCurrentPt.x,
				ptCurrentPt.y,
				ptCurrentPt.x + groupSize.cx,
				ptCurrentPt.y + groupSize.cy
			};

		RECT titleRect = 
		{
			ptCurrentPt.x + 10 ,
			ptCurrentPt.y - 20,
			ptCurrentPt.x + groupSize.cx,
			ptCurrentPt.y
		};

		DrawTitle(hDC, m_pList->GetCategoryName((SongAttribute)category), &titleRect);
		DrawGroupBox(hDC, &groupRect, nDividerOffset);
		
		DrawSongs(hDC, (SongAttribute)category, &groupRect, nDividerOffset);

		ptCurrentPt.x+=groupSize.cx + spaceBetween.cx;
		if (ptCurrentPt.x + groupSize.cx > m_nWidth)
		{
			//move down
			ptCurrentPt.x = ptUpperLeft.x;
			ptCurrentPt.y = ptCurrentPt.y + groupSize.cy + spaceBetween.cy;
		}

	}
	
}

void CPluginWindow::DrawGroupBox(HDC hDC, const LPRECT pRect, int nDividerOffset)
{
	HBRUSH hNullBrush = (HBRUSH)GetStockObject(NULL_BRUSH);

	HPEN hOldPen = (HPEN)SelectObject(hDC, hGroupBorderPen);
	HBRUSH hOldBrush = (HBRUSH)SelectObject(hDC, hNullBrush);

	::RoundRect(hDC, 
		pRect->left, 
		pRect->top, 
		pRect->right, 
		pRect->bottom, 
		40,40
		);


	//draw dividing line
	SelectObject(hDC, hDividerPen);
	MoveToEx(hDC, nDividerOffset+pRect->left, pRect->top + 5, NULL);
	LineTo(hDC, nDividerOffset+pRect->left, pRect->bottom - 5);

	SelectObject(hDC, hOldBrush);
	SelectObject(hDC, hOldPen);
}

void CPluginWindow::DrawTitle(HDC hDC, const CString& strTitle, const LPRECT pRect)
{
	int nOldDC = SaveDC(hDC);
	HBRUSH hNullBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	SelectObject(hDC, hNullBrush);

	::SelectObject(hDC, hGroupTitleFont);
	::SetTextColor(hDC, RGB(255,255,255));
	::SetBkMode(hDC, TRANSPARENT);
	::DrawText(hDC, strTitle, strTitle.GetLength(),pRect,0);

	RestoreDC(hDC, nOldDC);
}

void CPluginWindow::DrawSongs(HDC hDC, SongAttribute nCategory, const LPRECT pRect, int nDividerOffset)
{

	const int nLeftMargin = 5;
	const int nRightMargin = 5;
	const int nTopMargin = 10;
	const int yStep = 15;

	int nOldDC = SaveDC(hDC);
	HBRUSH hNullBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	SelectObject(hDC, hNullBrush);

	::SelectObject(hDC, hListFont);
	::SetTextColor(hDC, RGB(255,255,255));
	::SetBkMode(hDC, TRANSPARENT);
	
	int numValues = m_pList->GetNumValues();
	
	
	RECT destRect = 
	{
		pRect->left +nLeftMargin, 
		pRect->top + nTopMargin,
		pRect->left + nDividerOffset - 2,
		pRect->top+nTopMargin+yStep
	};

	RECT numberDestRect = 
	{
		pRect->left + nDividerOffset + 2,
		pRect->top + nTopMargin,
		pRect->right - nRightMargin,
		pRect->top+nTopMargin + yStep,
	};

	for (int i=0;i<numValues;i++)
	{
		CString strValue = m_pList->GetValueName(nCategory,i);
		if (strValue.GetLength() > 0)
		{
		
			CString strFormatted;
			strFormatted.Format(_T("%s"),strValue);
			
			::DrawText(hDC,strFormatted,strFormatted.GetLength(),&destRect,DT_LEFT | DT_VCENTER | DT_END_ELLIPSIS);
			
			CString strCount = CUtils::FormatIntWithCommas(m_pList->GetValueCount(nCategory, i));
									
			DrawText(hDC, strCount, strCount.GetLength(),&numberDestRect,DT_RIGHT | DT_VCENTER);
		}
		else
		{
			break;//no more values
		}
		numberDestRect.top+=yStep;
		numberDestRect.bottom+=yStep;
		destRect.top+=yStep;
		destRect.bottom+=yStep;
	}

	RestoreDC(hDC, nOldDC);
}

HFONT CPluginWindow::MakeFont(LPCTSTR szFontName, int nHeight, int nWeight)
{
	_ASSERT(szFontName);

	LOGFONT lf;
	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = -nHeight;
	lf.lfWeight = nWeight;
	lf.lfItalic = 0;
	lf.lfStrikeOut = 0;
	lf.lfEscapement = 0;
	lf.lfUnderline = 0;
	lf.lfCharSet = 0;
	lf.lfPitchAndFamily = 0;

	HRESULT hResult = StringCchCopy(
		lf.lfFaceName,
		LF_FACESIZE,
		szFontName
	);

	_ASSERT(hResult==S_OK);
	
	return ::CreateFontIndirect(&lf);
}

LRESULT CPluginWindow::OnLButtonUp(UINT nMsg, WPARAM wParam, 
		   LPARAM lParam, BOOL& bHandled)
{
	POINT pt =
	{
		LOWORD(lParam),
		HIWORD(lParam)
	};

	if (IsMouseOverUrl(pt))
	{
		//launch url
		CString strUrl;
		strUrl.LoadString(IDS_URL);
		ShellExecute(NULL, _T("open"), strUrl, NULL, NULL, SW_SHOWNORMAL);
	}
	return 0;
}

bool CPluginWindow::IsMouseOverUrl(const POINT& pt) const
{
	return (pt.x >=urlRect.left && pt.x <=urlRect.right &&
		pt.y >=urlRect.top && pt.y <=urlRect.bottom);
}
