/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#pragma once

#include <strsafe.h>
#include "atlwin.h"
#include "..\WMTop10Server\SharedTop10List.h"
#include "Resource.h"

class CWMTop10;

class CPluginWindow : public CWindowImpl<CPluginWindow>
{
public:

	BEGIN_MSG_MAP(CPluginWindow)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
		MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_SETCURSOR, OnSetCursor)
		MESSAGE_HANDLER(WM_LBUTTONUP, OnLButtonUp)
	END_MSG_MAP()

    CPluginWindow(CWMTop10 *pPlugin);
    

	LRESULT OnEraseBackground(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

    LRESULT OnPaint(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

	LRESULT OnCreate(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

	LRESULT OnDestroy(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

	LRESULT OnSize(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

	LRESULT OnSetCursor(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);

	LRESULT OnLButtonUp(UINT nMsg, WPARAM wParam, 
			   LPARAM lParam, BOOL& bHandled);
	
private:
	void DrawGroups(HDC hDC);
	void DrawTitle(HDC hDC, const CString& strTitle, const LPRECT pRect);
	void DrawGroupBox(HDC hDC, const LPRECT pRect, int nDividerOffset);
	void DrawSongs(HDC hDC, SongAttribute category, const LPRECT pRect, int nDividerOffset);
	void DrawMainTitle(HDC hDC);
	void DrawUrl(HDC hDC);
	void DrawErrorMessage(HDC hDC, const CString& strError);
	static HFONT MakeFont(LPCTSTR szFontName, int nHeight, int nWeight);

	void MoveUrl();
	bool IsMouseOverUrl(const POINT& pt) const;

private:
    CWMTop10  *m_pPlugin;  // pointer to plugin object
	const CSharedTop10List* m_pList ;//pointer to shared top 10 list

	int m_nWidth, m_nHeight;

	//fonts
	HFONT hTitleFont;
	HFONT hGroupTitleFont;
	HFONT hListFont;

	//brushes
	HBRUSH hBackgroundBrush;

	//pens
	HPEN hGroupBorderPen;
	HPEN hDividerPen;

	//static control link to web-site
	CString m_strUrl;
	CString m_strUrlDisplay;
	RECT urlRect;
};

