/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#include "stdafx.h"
#include "WMTop10.h"
#include "CPluginWindow.h"
#include "wmsdkidl.h"



/////////////////////////////////////////////////////////////////////////////
// CWMTop10::CWMTop10
// Constructor

CWMTop10::CWMTop10()
:m_hThread(NULL)
,m_hShutdownEvent(NULL)
,m_hRefreshEvent(NULL)
,m_hServerShutdownEvent(NULL)
, m_bServerRunning(false)
{
    m_pPluginWindow = NULL;
    lstrcpyn(m_szPluginText, _T("WMTop10 Plugin"), sizeof(m_szPluginText) / sizeof(m_szPluginText[0]));
    m_dwAdviseCookie = 0;
	m_pSharedList = new CSharedTop10List(true);
	
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::~CWMTop10
// Destructor

CWMTop10::~CWMTop10()
{
	delete m_pSharedList;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10:::FinalConstruct
// Called when an plugin is first loaded. Use this function to do one-time
// intializations that could fail instead of doing this in the constructor,
// which cannot return an error.

HRESULT CWMTop10::FinalConstruct()
{
	
	//grab events that our listening thread will need to listen to

	m_hShutdownEvent = CreateEvent(
		NULL,			//no security attributes
		TRUE,			//manual reset event
		FALSE,			//unsignaled at first
		gszShutdownEventName//global event name
		);

	m_hRefreshEvent = CreateEvent(
		NULL,			//no security attributes
		FALSE,			//auto-reset event
		FALSE,			//unsignaled at first
		gszRefreshEventName //global event name (Created in server process)
		);

	m_hServerShutdownEvent = CreateEvent(
		NULL,			//no security attributes
		FALSE,			//auto-reset event
		FALSE,			//unsignaled
		gszServerShutdownEventName//global event name (Created in server process)
		);

	_ASSERT(m_hRefreshEvent);
	_ASSERT(m_hShutdownEvent);
	_ASSERT(m_hServerShutdownEvent);

	if (m_hRefreshEvent && m_hShutdownEvent)
	{
		//start message listening thread
		m_hThread = CreateThread(
			NULL,		//security attributes
			0,			//stack size
			MessageListeningThread,//thread function
			this,		//thread function argument
			0,			//don't create suspended
			NULL);		//thread id (not needed in this case)
		_ASSERT(m_hThread);
	}

	//see if server is running
	HANDLE hServerMutex = OpenMutex(SYNCHRONIZE,
		FALSE,gszServerMutexName);
	if (hServerMutex!=NULL)
	{
		ServerRunning(true);
	}

    return S_OK;
}



/////////////////////////////////////////////////////////////////////////////
// CWMTop10:::FinalRelease
// Called when a plugin is unloaded. Use this function to free any
// resources allocated in FinalConstruct.

void CWMTop10::FinalRelease()
{
	SetEvent(m_hShutdownEvent);
	CloseHandle(m_hShutdownEvent);
    ReleaseCore();
}

//////////////////////////////////////////////////////////////////////
// CWMTop10::MessageListeningThread
// static function to call member function
DWORD WINAPI CWMTop10::MessageListeningThread(LPVOID pObj)
{
	CWMTop10* pThis = (CWMTop10*)pObj;
	_ASSERT(pThis);
	if (pThis)
	{
		return pThis->MessageListeningThread_Impl(NULL);
	}
	return 1;
}
//////////////////////////////////////////////////////////////////////
// CWMTop10::MessageListeningThread_Impl
// This function runs on a thread to listen for events from other plugin
DWORD CWMTop10::MessageListeningThread_Impl(LPVOID)
{
	HANDLE handles[] = {m_hShutdownEvent, m_hRefreshEvent, m_hServerShutdownEvent};
	bool bQuit = false;
	while (!bQuit)
	{
		DWORD dwResult = WaitForMultipleObjects(
			sizeof(handles)/sizeof(handles[0]),
			handles,
			FALSE,		//don't wait for all
			INFINITE);	//don't timeout
		
		switch (dwResult)
		{
		case WAIT_OBJECT_0:
			//shutdown
			bQuit = true;
			break;
		case WAIT_OBJECT_0+1:
			//refresh
			//also, server must be running
			
			ServerRunning(true);
			if (m_pPluginWindow && m_pPluginWindow->IsWindow())
			{
				m_pPluginWindow->RedrawWindow();
			}
		
			break;
		case WAIT_OBJECT_0+2:
			//server is shutting down
			ServerRunning(false);

			if (m_pPluginWindow && m_pPluginWindow->IsWindow())
			{
				m_pPluginWindow->RedrawWindow();
			}
			
			break;
		
		default:
			_ASSERT(0);
			break;
		}
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::SetCore
// Set WMP core interface

HRESULT CWMTop10::SetCore(IWMPCore *pCore)
{
    HRESULT hr = S_OK;

    // release any existing WMP core interfaces
    ReleaseCore();

    // If we get passed a NULL core, this  means
    // that the plugin is being shutdown.

    if (pCore == NULL)
    {
        return S_OK;
    }

    m_spCore = pCore;

    // connect up the event interface
    CComPtr<IConnectionPointContainer>  spConnectionContainer;

    hr = m_spCore->QueryInterface( &spConnectionContainer );

    if (SUCCEEDED(hr))
    {
        hr = spConnectionContainer->FindConnectionPoint( __uuidof(IWMPEvents), &m_spConnectionPoint );
    }

    if (SUCCEEDED(hr))
    {
        hr = m_spConnectionPoint->Advise( GetUnknown(), &m_dwAdviseCookie );

        if ((FAILED(hr)) || (0 == m_dwAdviseCookie))
        {
            m_spConnectionPoint = NULL;
        }
    }

    return hr;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::ReleaseCore
// Release WMP core interfaces

void CWMTop10::ReleaseCore()
{
    if (m_spConnectionPoint)
    {
        if (0 != m_dwAdviseCookie)
        {
            m_spConnectionPoint->Unadvise(m_dwAdviseCookie);
            m_dwAdviseCookie = 0;
        }
        m_spConnectionPoint = NULL;
    }

    if (m_spCore)
    {
        m_spCore = NULL;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::Create
// Create window for plugin

HRESULT CWMTop10::Create(HWND hwndParent, HWND *phwndWindow)
{
    if (NULL == phwndWindow)
    {
        return E_UNEXPECTED;
    }

    m_pPluginWindow = new CPluginWindow(this);
    if (NULL == m_pPluginWindow)
    {
        return E_OUTOFMEMORY;
    }

    // create the plugin window
    RECT rcPos = { 0, 0, 100, 100};
    HWND hWnd = m_pPluginWindow->Create(hwndParent, rcPos, _T("WMTop10 Plugin"));
    if (NULL == hWnd)
    {
        delete m_pPluginWindow;
        m_pPluginWindow = NULL;
        return E_FAIL;
    }

    *phwndWindow = hWnd;


    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::Destroy
// Destroy window for plugin

HRESULT CWMTop10::Destroy()
{
    if (NULL == m_pPluginWindow)
    {
        return E_UNEXPECTED;
    }

    m_pPluginWindow->DestroyWindow();
    delete m_pPluginWindow;
    m_pPluginWindow = NULL;

    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::TranslateAccelerator
// Handle TranslateAccelerator for plugin

HRESULT CWMTop10::TranslateAccelerator(LPMSG lpmsg)
{
    if (NULL == m_pPluginWindow)
    {
        return E_UNEXPECTED;
    }

    if (NULL == lpmsg)
    {
        return E_POINTER;
    }

    return S_OK;
}



/////////////////////////////////////////////////////////////////////////////
// CWMTop10::GetProperty
// Get plugin property

HRESULT CWMTop10::GetProperty(const WCHAR *pwszName, VARIANT *pvarProperty)
{

    if (NULL == pvarProperty)
    {
        return E_POINTER;
    }

    HRESULT hr = S_OK;

    if (0 == lstrcmpiW(pwszName, PLUGIN_SEPARATEWINDOW_DEFAULTWIDTH ))
    {
        pvarProperty->vt = VT_I4;
        pvarProperty->lVal = 100;
    }
    else if (0 == lstrcmpiW(pwszName, PLUGIN_SEPARATEWINDOW_DEFAULTHEIGHT))
    {
        pvarProperty->vt = VT_I4;
        pvarProperty->lVal = 100;
    }
    else
    {
        hr = E_NOTIMPL;
    }

    return hr;
    
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::SetProperty
// Set plugin property

HRESULT CWMTop10::SetProperty(const WCHAR *pwszName, const VARIANT *pvarProperty)
{
    return E_NOTIMPL;
}


