/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/

#include "atlwin.h"
#include "SettingsStruct.h"
#include "WMTop10Server.h"

class CPropertyDialog : public CDialogImpl<CPropertyDialog>
{
public:
    enum { IDD = IDD_PROPERTYDIALOG };

	BEGIN_MSG_MAP(CPropertyDialog)
        MESSAGE_HANDLER( WM_INITDIALOG, OnInitDialog )
        COMMAND_ID_HANDLER( IDOK, OnOK )
        COMMAND_ID_HANDLER( IDCANCEL, OnCancel )
		COMMAND_ID_HANDLER( IDC_CLEARALL, OnClearAll)
		COMMAND_ID_HANDLER( IDC_LAUNCHSITE, OnLaunchSite)
	END_MSG_MAP()

    CPropertyDialog(SettingsStruct& rProgramSettings, CWMTop10Server* pServer)
	:rSettings(rProgramSettings)
	, m_pServer(pServer)
    {
        
    }

    LRESULT OnInitDialog( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& fHandled )
	{
        CenterWindow();

		
		this->CheckDlgButton(IDC_MARKUNKNOWNS, rSettings.bTrackUnknowns?BST_CHECKED:BST_UNCHECKED);
		switch(rSettings.nRecordSignal)
		{
		case rssSongStart:
			CheckRadioButton(IDC_SONGSTARTS, IDC_SONGENDS, IDC_SONGSTARTS);
			break;
		case rssSongEnd:
			CheckRadioButton(IDC_SONGSTARTS, IDC_SONGENDS, IDC_SONGENDS);
			break;
		default:
			_ASSERT(0);
			break;
		}
		
        
		return 1;
	}

	
    LRESULT OnClearAll(WORD wNotifyCode, WORD wID, HWND hwndCtl, BOOL& fHandled)
	{
		CString strMessage = _T("Are you sure you want to clear all your playback information?");
		if (::MessageBox(this->m_hWnd, strMessage,_T("Please Confirm"),MB_YESNO | MB_ICONQUESTION)==IDYES)
		{
			m_pServer->ClearDB();
		}
		return 0;
	}

    LRESULT OnOK(WORD wNotifyCode, WORD wID, HWND hwndCtl, BOOL& fHandled)
	{
        // save options
		BOOL bMarkUnknowns = (IsDlgButtonChecked(IDC_MARKUNKNOWNS)==BST_CHECKED);
		
		BOOL bSongStarts = (IsDlgButtonChecked(IDC_SONGSTARTS)==BST_CHECKED);
		BOOL bSongEnds = (IsDlgButtonChecked(IDC_SONGENDS)==BST_CHECKED);

		//make sure only one of the two is checked
		_ASSERT((bSongStarts ^ bSongEnds)==1);
		if (!((bSongStarts ^ bSongEnds)==1))
		{
			::MessageBox(this->m_hWnd,_T("Either \"Song Starts\" or \"Song Ends\" must be checked, but not both"),_T("Error"),MB_OK | MB_ICONERROR);
		}
		else
		{
			//save to settings struct
			rSettings.bTrackUnknowns = (bMarkUnknowns==TRUE);
			if (bSongStarts) {
				rSettings.nRecordSignal = rssSongStart;
			}
			else if (bSongEnds)
			{
				rSettings.nRecordSignal = rssSongEnd;
			}
			else
			{
				_ASSERT(0);
			}

			EndDialog( IDOK );
		}
		return 0;
	}

    LRESULT OnCancel(WORD wNotifyCode, WORD wID, HWND hwndCtl, BOOL& fHandled)
	{
        EndDialog( IDCANCEL );
		return 0;
	}

	LRESULT OnLaunchSite(WORD wNotifyCode, WORD wID, HWND hwndCtl, BOOL& fHandled)
	{
		CString strLaunch;
		strLaunch.LoadString(IDS_URL);
		ShellExecute(NULL, _T("open"), strLaunch, NULL, NULL, SW_SHOWNORMAL);

		return 0;
	}

private:
    SettingsStruct& rSettings;
	CWMTop10Server* m_pServer;
};

