/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#include "StdAfx.h"
#include <strsafe.h>
#include "SharedTop10List.h"

CSharedTop10List::CSharedTop10List(bool bReadOnly)
:m_hFile(0)
,m_pBase(NULL)
,m_bReadOnly(bReadOnly)
,m_szMapName(_T("WMTOP10LIST"))
{
	CreateSharedMemory();
}

CSharedTop10List::~CSharedTop10List(void)
{
	DestroySharedMemory();
}

void CSharedTop10List::CreateSharedMemory(void)
{
	m_hFile = CreateFileMapping(
		INVALID_HANDLE_VALUE,	//page-file only
		NULL,					//no security attributes
		PAGE_READWRITE,			//access
		0,						//max size (high)
		sizeof(TOP10STRUCT),	//max size (low)
		m_szMapName);
	
	_ASSERT(m_hFile!=NULL);

	if (m_hFile==NULL)
	{
		return;
	}

	m_pBase = (TOP10STRUCT*)MapViewOfFile(
		m_hFile,								//handle to file
		m_bReadOnly?FILE_MAP_READ:FILE_MAP_WRITE,	//access
		0,										//offset (high)
		0,										//offset (low)
		sizeof(TOP10STRUCT)						//size
		);	

#ifdef _DEBUG
	_ASSERT(m_pBase!=NULL);
	if (m_pBase==NULL){
		int nError = GetLastError();
		_tprintf(_T("Last error: %d"),nError);
	}
#endif

	//initialize fields
	if (m_pBase!=NULL && !m_bReadOnly)
	{
		m_pBase->nFieldLength=FIELDLENGTH;
		m_pBase->nNumCategories=NUMCATEGORIES;
		m_pBase->nNumValues=NUMVALUES;
	}


}

void CSharedTop10List::DestroySharedMemory(void)
{
	UnmapViewOfFile(m_pBase);
	CloseHandle(m_hFile);
}

//////////////////////////////////////////////////////////////////////////////////////
// Function: GetCategoryName
// Description: gets the shared list's category name
//////////////////////////////////////////////////////////////////////////////////////
LPCTSTR CSharedTop10List::GetCategoryName(SongAttribute nCategory) const
{
	if (nCategory>=0 && nCategory < saNumSongAttributes)
	{
		return m_pBase->categories[nCategory].szCategoryName;
	}
	else
	{
		_ASSERT(0);
		return NULL;
	}
}
//////////////////////////////////////////////////////////////////////////////////////
// Function: GetValueName
// Description: gets the value name for a specific entry in a category
//////////////////////////////////////////////////////////////////////////////////////
LPCTSTR CSharedTop10List::GetValueName(SongAttribute nCategory, int nIndex) const
{
	if (nCategory>=0 && nCategory < saNumSongAttributes)
	{
		return m_pBase->categories[nCategory].values[nIndex].szValue;
	}
	else
	{
		_ASSERT(0);
		return NULL;
	}
}

int CSharedTop10List::GetValueCount(SongAttribute nCategory, int nIndex) const
{
	if (nCategory>=0 && nCategory < saNumSongAttributes)
	{
		return m_pBase->categories[nCategory].values[nIndex].nCount;
	}
	else
	{
		_ASSERT(0);
		return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// Function: UpdateCategoryName
// Description: updates the shared list's category name
//////////////////////////////////////////////////////////////////////////////////////
void CSharedTop10List::UpdateCategoryName(SongAttribute nCategory, const CString& strName)
{
	if (m_pBase!=NULL)
	{
		if (nCategory < (int)m_pBase->nNumCategories)
		{
			HRESULT hResult = StringCchCopy(
				m_pBase->categories[nCategory].szCategoryName,
				m_pBase->nFieldLength,
				strName
				);

			_ASSERT(hResult==S_OK || hResult==STRSAFE_E_INSUFFICIENT_BUFFER);
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////
// Function: UpdateEntry
// Description: updates the shared list with a new value
//////////////////////////////////////////////////////////////////////////////////////
void CSharedTop10List::UpdateEntry(SongAttribute nCategory, int nIndex, int nCount, const CString& strValue)
{
	
	if (m_pBase!=NULL)
	{
		if (nCategory < (int)m_pBase->nNumCategories 
			&& nIndex < (int)m_pBase->nNumValues)
		{
			HRESULT hResult = StringCchCopy(
				m_pBase->categories[nCategory].values[nIndex].szValue,
				m_pBase->nFieldLength,
				strValue
				);

			_ASSERT(hResult==S_OK || hResult==STRSAFE_E_INSUFFICIENT_BUFFER);
			m_pBase->categories[nCategory].values[nIndex].nCount = nCount;

		}

	}
	
}

void CSharedTop10List::Clear()
{
	if (m_pBase!=NULL)
	{
		for (unsigned int i=0;i<m_pBase->nNumCategories;i++)
		{
			ZeroMemory(&m_pBase->categories[i], sizeof(m_pBase->categories[i]));
			
		}
	}
}