/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#pragma once
#include "const.h"
#include "OrderedList.h"
#include "ValueCountPair.h"
#include "ValueCounts.h"

#pragma pack(push,1)
struct DBHEADERSTRUCT
{
	BYTE nVersion;
	DWORD dwNumSections;
};

struct DBSECTIONHEADERSTRUCT
{
	BYTE nSectionId;
	DWORD dwNumTop10Values;
	DWORD dwNumValues;
};

struct DBVALUESTRUCT
{
	DWORD cchValueLength;
	DWORD dwCount;
};

#pragma pack(pop)

class CTop10Database
{
public:
	CTop10Database(void);
	~CTop10Database(void);

	void AddSong(const vector<CString> strValues[saNumSongAttributes]);
	CValueCountPair& GetTopValue(SongAttribute nCategory, int nIndex);
	int GetNumValues(SongAttribute nCategory) const ;
	int GetNumCategories() const { return saNumSongAttributes; }

	bool Load();
	bool Save();
	void Clear();
protected:
	CString GetDBFilename(void) const;
	bool IsDirty() const { return m_bDirty; }
	void SetDirty() { m_bDirty=true;}
	void ClearDirty() { m_bDirty = false;}
private:
	bool LoadVersion1(HANDLE hFile);

	bool WriteValueCountPair(HANDLE hFile, const CValueCountPair& rPair) const;
	bool ReadValueCountPair(HANDLE hFile, CValueCountPair& rPair) const;
	
	
private:
	COrderedList m_topLists[saNumSongAttributes];
	CValueCounts m_masterList[saNumSongAttributes];
	
	static const BYTE m_nDBVersion;
	static const int m_nValueLengthLimit;
	bool m_bDirty;

	
};
