/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#pragma once
#include <hash_map>
#include "ValueCountPair.h"

using namespace stdext;

class CaseInsensitiveCompare
{
    public:
        bool operator()(LPCTSTR x, LPCTSTR y) const
        {
			if (_tcsicmp(x,y) > 0)
				return true;
			return false;
        }
};

typedef hash_map<LPCTSTR, CValueCountPair, hash_compare<LPCTSTR, CaseInsensitiveCompare> > ValueCountMap;
typedef ValueCountMap::const_iterator ValueCountMapConstIterator;
typedef ValueCountMap::iterator ValueCountMapIterator;

class CValueCounts;

class CValueCountsIterator
{
public:
	void First();
	bool Next(CValueCountPair& rPair);

private:
	CValueCountsIterator(const ValueCountMap& map)
		:m_rMap(map)
		,m_bReset(true)
	{
		
	}

	ValueCountMapConstIterator m_iter;
	const ValueCountMap& m_rMap;
	bool m_bReset;
	friend CValueCounts;
};	

class CValueCounts
{
public:
	CValueCounts(void);
	~CValueCounts(void);
	const CValueCountPair& AddValue(const CString& rValue);
	void AddValue(const CValueCountPair& rValueCounts);
	void Clear();
	size_t NumItems() const { return (size_t)m_map.size(); }
	CValueCountsIterator GetIterator() { return CValueCountsIterator(m_map); }
	CValueCountsIterator GetIterator() const { return CValueCountsIterator(m_map); }
private:
	ValueCountMap m_map;
	
		

};
