/*
http://www.philosophicalgeek.com/software/windows-media-top-10-plug-in/
Copyright (c) 2007 Ben Watson (http://www.philosophicalgeek.com)

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this
software in a product, an acknowledgment in the product documentation
would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and
must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/

#include "stdafx.h"
#include "WMTop10Server.h"
#include "CPropertyDialog.h"

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::CWMTop10Server
// Constructor

CWMTop10Server::CWMTop10Server()
: m_hRefreshEvent(NULL)
, m_hServerShutdownEvent(NULL)
, m_hServerMutex(NULL)
{
    m_dwAdviseCookie = 0;

	m_pDB = new CTop10Database();

	m_pSharedList = new CSharedTop10List(false);
	
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::~CWMTop10Server
// Destructor

CWMTop10Server::~CWMTop10Server()
{
	delete m_pDB;
	delete m_pSharedList;	
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server:::FinalConstruct
// Called when an plugin is first loaded. Use this function to do one-time
// intializations that could fail instead of doing this in the constructor,
// which cannot return an error.

HRESULT CWMTop10Server::FinalConstruct()
{
	Initialize();

	bool bSuccess = m_pDB->Load();
	_ASSERT(bSuccess);
	UpdateSharedList();

	m_hServerMutex = CreateMutex(
		NULL,			//no security attributes
		TRUE,			//claim ownership immediately
		gszServerMutexName//global event name
		);

	m_hRefreshEvent = CreateEvent(
		NULL,			//no security attributes
		FALSE,			//auto-reset event
		FALSE,			//unsignaled at first
		gszRefreshEventName //global event name (Created in server process)
		);

	m_hServerShutdownEvent = CreateEvent(
		NULL,			//no security attributes
		FALSE,			//auto-reset event
		FALSE,			//unsignaled at first
		gszServerShutdownEventName //global event name (Created in server process)
		);
	
	_ASSERT(m_hRefreshEvent);
	_ASSERT(m_hServerShutdownEvent);

	SetEvent(m_hRefreshEvent);

    return S_OK;
}


/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server:::FinalRelease
// Called when a plugin is unloaded. Use this function to free any
// resources allocated in FinalConstruct.

void CWMTop10Server::FinalRelease()
{
	bool bSuccess = m_pDB->Save();
	_ASSERT(bSuccess);

	SetEvent(m_hServerShutdownEvent);

	CloseHandle(m_hRefreshEvent);
	CloseHandle(m_hServerShutdownEvent);
    
	CloseHandle(m_hServerMutex);

	ReleaseCore();

	
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10::DisplayPropertyPage
// Display property page for plugin

HRESULT CWMTop10Server::DisplayPropertyPage(HWND hwndParent)
{
    CPropertyDialog dialog(settings, this);

    INT_PTR nResult = dialog.DoModal(hwndParent);
	if (nResult==IDOK)
	{
		//settings were changed
		//no need to do anything
	}

    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::SetCore
// Set WMP core interface

HRESULT CWMTop10Server::SetCore(IWMPCore *pCore)
{
    HRESULT hr = S_OK;

    // release any existing WMP core interfaces
    ReleaseCore();

    // If we get passed a NULL core, this  means
    // that the plugin is being shutdown.

    if (pCore == NULL)
    {
        return S_OK;
    }

    m_spCore = pCore;

    // connect up the event interface
    CComPtr<IConnectionPointContainer>  spConnectionContainer;

    hr = m_spCore->QueryInterface( &spConnectionContainer );

    if (SUCCEEDED(hr))
    {
        hr = spConnectionContainer->FindConnectionPoint( __uuidof(IWMPEvents), &m_spConnectionPoint );
    }

    if (SUCCEEDED(hr))
    {
        hr = m_spConnectionPoint->Advise( GetUnknown(), &m_dwAdviseCookie );

        if ((FAILED(hr)) || (0 == m_dwAdviseCookie))
        {
            m_spConnectionPoint = NULL;
        }
    }

    return hr;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::ReleaseCore
// Release WMP core interfaces

void CWMTop10Server::ReleaseCore()
{
    if (m_spConnectionPoint)
    {
        if (0 != m_dwAdviseCookie)
        {
            m_spConnectionPoint->Unadvise(m_dwAdviseCookie);
            m_dwAdviseCookie = 0;
        }
        m_spConnectionPoint = NULL;
    }

    if (m_spCore)
    {
        m_spCore = NULL;
    }
}





/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::GetProperty
// Get plugin property

HRESULT CWMTop10Server::GetProperty(const WCHAR *pwszName, VARIANT *pvarProperty)
{

	    if (NULL == pvarProperty)
    {
        return E_POINTER;
    }

    
    return E_NOTIMPL;
}

/////////////////////////////////////////////////////////////////////////////
// CWMTop10Server::SetProperty
// Set plugin property

HRESULT CWMTop10Server::SetProperty(const WCHAR *pwszName, const VARIANT *pvarProperty)
{
    return E_NOTIMPL;
}

void CWMTop10Server::Initialize()
{
	
	m_WMAttribNames[saTitle] = g_wszWMTitle;
	m_WMAttribNames[saAlbumTitle] = g_wszWMAlbumTitle;
	m_WMAttribNames[saAlbumArtist] = g_wszWMAlbumArtist;
	m_WMAttribNames[saComposers] = g_wszWMComposer;
	m_WMAttribNames[saArtists] = g_wszWMAuthor;
	m_WMAttribNames[saGenres] = g_wszWMGenre;
	m_WMAttribNames[saLanguages] = g_wszWMLanguage;
}

void CWMTop10Server::AddCurrentSong()
{
	_ASSERT(m_pDB);
	if (m_pDB)
	{
		CComPtr<IWMPMedia> spMedia;
		CComPtr<IWMPMedia3> spMedia3;
		
				
		if (m_spCore)
		{
			HRESULT hr = m_spCore->get_currentMedia(&spMedia);
			HRESULT hr3 = spMedia.QueryInterface(&spMedia3);

			if (SUCCEEDED(hr) && SUCCEEDED(hr3) && spMedia && spMedia3)
			{
				//ignore everything but audio
				if (!IsCurrentMediaAudio(spMedia))
				{
					return;
				}

				vector<CString>* pAttributes = new vector<CString>[saNumSongAttributes];
				const WCHAR* pwszAttribName  = NULL;
				bool bTitleFound = false, bArtistFound = false;
				//get all attributes
				for (int attribute=0;attribute<saNumSongAttributes; attribute++)
				{
					pwszAttribName = m_WMAttribNames[attribute];

					bool bSuccess = false;
					switch(attribute)
					{
					case saTitle:
					case saAlbumTitle:
						bSuccess = GetSongProperty(spMedia, pwszAttribName, pAttributes[attribute]);
						break;

					case saAlbumArtist:
					case saComposers:
					case saArtists:
					case saGenres:
					case saLanguages:
						bSuccess = GetSongMultiProperty(spMedia3, pwszAttribName, pAttributes[attribute]);
						break;
					default:
						
						//do nothing
						break;
					}
					
					if (bSuccess)
					{
						if (attribute==saTitle)
						{
							bTitleFound = true;
						}
						if (attribute==saAlbumArtist)
						{
							bArtistFound = true;
						}
					}
					
					
				}

				//add artist to title
				if (bTitleFound && bArtistFound)
				{
					CString append;
					if (pAttributes[saAlbumArtist][0].GetLength() > 0)
					{
						append.Format(_T(" (%s)"), pAttributes[saAlbumArtist][0]);
						pAttributes[saTitle][0]+=append;
					}
				}
				m_pDB->AddSong(pAttributes);
				UpdateSharedList();
			}
		}
	}
}

bool CWMTop10Server::IsCurrentMediaAudio(CComPtr<IWMPMedia> spMedia)
{
	LPCTSTR szGuidMusic = _T("{D1607DBC-E323-4BE2-86A1-48A42A28441E}");
	LPCTSTR szGuidNonMusicAudio = _T("{01CD0F29-DA4E-4157-897B-6275D50C4F11}");
	
	CComBSTR bstrName = g_wszWMMediaClassPrimaryID;
	CComBSTR bstrValue;

	HRESULT hr = spMedia->getItemInfo(bstrName, &bstrValue);
	CString strVal;
	if (SUCCEEDED(hr))
	{
		strVal = COLE2T(bstrValue.m_str);
		strVal.Trim();
	}

	return (strVal==szGuidMusic || strVal==szGuidNonMusicAudio);
}

bool CWMTop10Server::GetSongProperty(CComPtr<IWMPMedia> spMedia, const TCHAR* wszAttribute, vector<CString>& rAttributes)
{
	_ASSERT(wszAttribute);
	_ASSERT(spMedia);

	CComBSTR bstrName = wszAttribute;
	CComBSTR bstrValue;

	bool bFound = true;

	HRESULT hr = spMedia->getItemInfo(bstrName, &bstrValue);
	CString strVal;
	if (SUCCEEDED(hr))
	{
		rAttributes.clear();
		strVal = COLE2T(bstrValue.m_str);
		strVal.Trim();
	}
	if (settings.bTrackUnknowns && strVal.GetLength() <=0) 
	{
		strVal = _T("<Unknown>");
		bFound = false;
	}

	rAttributes.push_back(strVal);

	return bFound;
}


bool CWMTop10Server::GetSongMultiProperty(CComPtr<IWMPMedia3> spMedia, const TCHAR* wszAttribute, vector<CString>& rAttributes)
{
	_ASSERT(wszAttribute);
	_ASSERT(spMedia);

	CComBSTR bstrName = wszAttribute;

	HRESULT hr = 0;
	long numValues = 0;
	//get the number of values
	hr = spMedia->getAttributeCountByType(bstrName,NULL, &numValues);
	if (FAILED(hr))
	{
		return false;
	}
	
	rAttributes.clear();
	
	bool bFound = true;

	for (int i=0;i<numValues;i++)
	{
		VARIANT vValue;
		hr = spMedia->getItemInfoByType(bstrName, NULL, i, &vValue);
		CString strVal;
		if (SUCCEEDED(hr))
		{
			_ASSERT(vValue.vt == VT_BSTR);
			if (vValue.vt==VT_BSTR)
			{
				strVal = COLE2T(vValue.bstrVal);
				strVal.Trim();
			}
			
		}
		if (settings.bTrackUnknowns && strVal.GetLength() <=0)
		{
			strVal = _T("<Unknown>");
			bFound = false;
		}

		rAttributes.push_back(strVal);
		
	}
	
	return bFound;
}

void CWMTop10Server::UpdateSharedList()
{
	_ASSERT(m_pSharedList!=NULL);
	
	m_pSharedList->Clear();
	
	m_pSharedList->UpdateCategoryName(saTitle, _T("Title"));
	m_pSharedList->UpdateCategoryName(saAlbumTitle , _T("Album Title"));
	m_pSharedList->UpdateCategoryName(saAlbumArtist , _T("Album Artist"));
	m_pSharedList->UpdateCategoryName(saComposers , _T("Composer"));
	m_pSharedList->UpdateCategoryName(saArtists , _T("Artist"));
	m_pSharedList->UpdateCategoryName(saGenres , _T("Genre"));
	m_pSharedList->UpdateCategoryName(saLanguages , _T("Language"));

	int numCategories = m_pDB->GetNumCategories();
	for (int category=0;category<numCategories;category++)
	{
		int numValues = m_pDB->GetNumValues((SongAttribute)category);
		for (int i=0;i<numValues;i++)
		{
			CValueCountPair& pair = m_pDB->GetTopValue((SongAttribute)category, i);
			m_pSharedList->UpdateEntry((SongAttribute)category, i, pair.Count(), pair.Value());
		}

		
	}

}

void CWMTop10Server::ClearDB()
{
	if (m_pDB!=NULL)
	{
		m_pDB->Clear();
		UpdateSharedList();

		//signal event to refresh UI plugin
		if (m_hRefreshEvent != NULL)
		{
			BOOL bSuccess = SetEvent(m_hRefreshEvent);
			_ASSERT(bSuccess);
		}
		
	}
}